{% extends "skeleton.html" %}
{% load autobreadcrumb %}

{% block head_title %}{% current_title_from_breadcrumbs %}{% endblock %}

{% block head_more_links %}{{ block.super }}
    {% include "datebook/include_css.html" %}
{% endblock %}

{% block head_base_js %}{{ block.super }}
    {% include "datebook/include_js.html" %}
{% endblock %}

{% block menu_active_link_datebook %} class="active"{% endblock %}

{% block base_content %}<div class="row datebook{% block datebook_container_css_classes %}{% endblock %}">
    <div class="small-12 columns">
        {% block breadcrumbs_block %}{% autobreadcrumbs_tag %}{% endblock %}
        {% block datebook_maintitle %}<h2>{% current_title_from_breadcrumbs %}</h2>{% endblock %}
        {% block datebook_content %}{% endblock %}
    </div>
</div>{% endblock %}
