{% load assets %}{% assets "datebook_app_js" %}
    <script type="text/javascript" src="{{ ASSET_URL }}"></script>
{% endassets %}
{% comment %}<script type="text/javascript" src="{{ STATIC_URL }}js/datebook_app.min.js"></script>{% endcomment %}