{% extends "datebook/base.html" %}
{% load i18n crispy_forms_tags %}

{% block head_base_js %}{{ block.super }}
    <script type="text/javascript">
    //<![CDATA[
    $(document).ready(function() {
        var picker = new Pikaday({
            field: $('#id_period')[0],
            //format: 'YYYY-MM-DD',
            format: 'DD/MM/YYYY',
            i18n: {
                months        : ['{% trans "January" %}','{% trans "February" %}','{% trans "March" %}','{% trans "April" %}','{% trans "May" %}','{% trans "June" %}','{% trans "July" %}','{% trans "August" %}','{% trans "September" %}','{% trans "October" %}','{% trans "November" %}','{% trans "December" %}'],
                weekdays      : ['{% trans "Sunday" %}','{% trans "Monday" %}','{% trans "Tuesday" %}','{% trans "Wednesday" %}','{% trans "Thursday" %}','{% trans "Friday" %}','{% trans "Saturday" %}'],
                weekdaysShort : ['{% trans "Sun" %}','{% trans "Mon" %}','{% trans "Tue" %}','{% trans "Wed" %}','{% trans "Thu" %}','{% trans "Fri" %}','{% trans "Sat" %}']
            }
        });
    });
    //]]>
    </script>
{% endblock %}

{% block datebook_content %}
    
{% crispy form %}
{% endblock %}